@@ -3,7 +3,7 @@
 /* ========================================================================== */
 
 /* -----------------------------------------------------------------------------
- * CHOLMOD/Cholesky Module.  Version 0.6.  Copyright (C) 2005, Timothy A. Davis
+ * CHOLMOD/Cholesky Module.  Copyright (C) 2005-2006, Timothy A. Davis
  * The CHOLMOD/Cholesky Module is licensed under Version 2.1 of the GNU
  * Lesser General Public License.  See lesser.txt for a text of the license.
  * CHOLMOD is also available under other licenses; contact authors for details.
@@ -62,11 +62,10 @@
 static void LSOLVE (PREFIX,1)
 (
     cholmod_factor *L,
-    cholmod_dense *Y		    /* n-by-1 in row form */
+    double X [ ]			/* n-by-1 in row form */
 )
 {
     double *Lx = L->x ;
-    double *X = Y->x ;
     Int *Li = L->i ;
     Int *Lp = L->p ;
     Int *Lnz = L->nz ;
@@ -94,12 +93,11 @@ static void LSOLVE (PREFIX,1)
 #elif defined (LD)
 	    X [j] = y / Lx [p] ;
 #endif
-
 	    for (p++ ; p < pend ; p++)
 	    {
 		X [Li [p]] -= Lx [p] * y ;
 	    }
-	    j++ ;
+	    j++ ;	/* advance to next column of L */
 
 	}
 	else if (lnz != Lnz [j+2] + 2 || Li [p+2] != j+2)
@@ -111,7 +109,6 @@ static void LSOLVE (PREFIX,1)
 
 	    double y [2] ;
 	    Int q = Lp [j+1] ;
-
 #ifdef LL
 	    y [0] = X [j] / Lx [p] ;
 	    y [1] = (X [j+1] - Lx [p+1] * y [0]) / Lx [q] ;
@@ -127,12 +124,11 @@ static void LSOLVE (PREFIX,1)
 	    y [1] = X [j+1] - Lx [p+1] * y [0] ;
 	    X [j+1] = y [1] ;
 #endif
-
 	    for (p += 2, q++ ; p < pend ; p++, q++)
 	    {
 		X [Li [p]] -= Lx [p] * y [0] + Lx [q] * y [1] ;
 	    }
-	    j += 2 ;
+	    j += 2 ;	    /* advance to next column of L */
 
 	}
 	else
@@ -166,12 +162,11 @@ static void LSOLVE (PREFIX,1)
 	    X [j+1] = y [1] ;
 	    X [j+2] = y [2] ;
 #endif
-
 	    for (p += 3, q += 2, r++ ; p < pend ; p++, q++, r++)
 	    {
 		X [Li [p]] -= Lx [p] * y [0] + Lx [q] * y [1] + Lx [r] * y [2] ;
 	    }
-	    j += 3 ;
+	    j += 3 ;	    /* advance to next column of L */
 	}
     }
 }
@@ -186,11 +181,10 @@ static void LSOLVE (PREFIX,1)
 static void LSOLVE (PREFIX,2)
 (
     cholmod_factor *L,
-    cholmod_dense *Y		    /* n-by-2 in row form */
+    double X [ ][2]		/* n-by-2 in row form */
 )
 {
     double *Lx = L->x ;
-    double *X = Y->x ;
     Int *Li = L->i ;
     Int *Lp = L->p ;
     Int *Lnz = L->nz ;
@@ -212,25 +206,24 @@ static void LSOLVE (PREFIX,2)
 	    /* -------------------------------------------------------------- */
 
 	    double y [2] ;
-	    y [0] = X [2*j  ] ;
-	    y [1] = X [2*j+1] ;
+	    y [0] = X [j][0] ;
+	    y [1] = X [j][1] ;
 #ifdef LL
 	    y [0] /= Lx [p] ;
 	    y [1] /= Lx [p] ;
-	    X [2*j  ] = y [0] ;
-	    X [2*j+1] = y [1] ;
+	    X [j][0] = y [0] ;
+	    X [j][1] = y [1] ;
 #elif defined (LD)
-	    X [2*j  ] = y [0] / Lx [p] ;
-	    X [2*j+1] = y [1] / Lx [p] ;
+	    X [j][0] = y [0] / Lx [p] ;
+	    X [j][1] = y [1] / Lx [p] ;
 #endif
-
 	    for (p++ ; p < pend ; p++)
 	    {
-		Int i = 2 * Li [p] ;
-		X [i  ] -= Lx [p] * y [0] ;
-		X [i+1] -= Lx [p] * y [1] ;
+		Int i = Li [p] ;
+		X [i][0] -= Lx [p] * y [0] ;
+		X [i][1] -= Lx [p] * y [1] ;
 	    }
-	    j++ ;
+	    j++ ;	/* advance to next column of L */
 
 	}
 	else if (lnz != Lnz [j+2] + 2 || Li [p+2] != j+2)
@@ -242,38 +235,37 @@ static void LSOLVE (PREFIX,2)
 
 	    double y [2][2] ;
 	    Int q = Lp [j+1] ;
-	    y [0][0] = X [2*j  ] ;
-	    y [0][1] = X [2*j+1] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
-	    y [1][0] = (X [2*j+2] - Lx [p+1] * y [0][0]) / Lx [q] ;
-	    y [1][1] = (X [2*j+3] - Lx [p+1] * y [0][1]) / Lx [q] ;
-	    X [2*j  ] = y [0][0] ;
-	    X [2*j+1] = y [0][1] ;
-	    X [2*j+2] = y [1][0] ;
-	    X [2*j+3] = y [1][1] ;
+	    y [1][0] = (X [j+1][0] - Lx [p+1] * y [0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx [p+1] * y [0][1]) / Lx [q] ;
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
 #elif defined (LD)
-	    y [1][0] = X [2*j+2] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [2*j+3] - Lx [p+1] * y [0][1] ;
-	    X [2*j  ] = y [0][0] / Lx [p] ;
-	    X [2*j+1] = y [0][1] / Lx [p] ;
-	    X [2*j+2] = y [1][0] / Lx [q] ;
-	    X [2*j+3] = y [1][1] / Lx [q] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
 #else
-	    y [1][0] = X [2*j+2] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [2*j+3] - Lx [p+1] * y [0][1] ;
-	    X [2*j+2] = y [1][0] ;
-	    X [2*j+3] = y [1][1] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
 #endif
-
 	    for (p += 2, q++ ; p < pend ; p++, q++)
 	    {
-		Int i = 2 * Li [p] ;
-		X [i  ] -= Lx [p] * y [0][0] + Lx [q] * y [1][0] ;
-		X [i+1] -= Lx [p] * y [0][1] + Lx [q] * y [1][1] ;
+		Int i = Li [p] ;
+		X [i][0] -= Lx [p] * y [0][0] + Lx [q] * y [1][0] ;
+		X [i][1] -= Lx [p] * y [0][1] + Lx [q] * y [1][1] ;
 	    }
-	    j += 2 ;
+	    j += 2 ;	    /* advance to next column of L */
 
 	}
 	else
@@ -286,50 +278,49 @@ static void LSOLVE (PREFIX,2)
 	    double y [3][2] ;
 	    Int q = Lp [j+1] ;
 	    Int r = Lp [j+2] ;
-	    y [0][0] = X [2*j  ] ;
-	    y [0][1] = X [2*j+1] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
-	    y [1][0] = (X[2*j+2] - Lx[p+1] * y[0][0]) / Lx [q] ;
-	    y [1][1] = (X[2*j+3] - Lx[p+1] * y[0][1]) / Lx [q] ;
-	    y [2][0] = (X[2*j+4] - Lx[p+2] * y[0][0] - Lx[q+1] * y[1][0])/Lx[r];
-	    y [2][1] = (X[2*j+5] - Lx[p+2] * y[0][1] - Lx[q+1] * y[1][1])/Lx[r];
-	    X [2*j  ] = y [0][0] ;
-	    X [2*j+1] = y [0][1] ;
-	    X [2*j+2] = y [1][0] ;
-	    X [2*j+3] = y [1][1] ;
-	    X [2*j+4] = y [2][0] ;
-	    X [2*j+5] = y [2][1] ;
+	    y [1][0] = (X [j+1][0] - Lx[p+1] * y[0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx[p+1] * y[0][1]) / Lx [q] ;
+	    y [2][0] = (X [j+2][0] - Lx[p+2] * y[0][0] - Lx[q+1]*y[1][0])/Lx[r];
+	    y [2][1] = (X [j+2][1] - Lx[p+2] * y[0][1] - Lx[q+1]*y[1][1])/Lx[r];
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
 #elif defined (LD)
-	    y [1][0] = X [2*j+2] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [2*j+3] - Lx [p+1] * y [0][1] ;
-	    y [2][0] = X [2*j+4] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [2*j+5] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    X [2*j  ] = y [0][0] / Lx [p] ;
-	    X [2*j+1] = y [0][1] / Lx [p] ;
-	    X [2*j+2] = y [1][0] / Lx [q] ;
-	    X [2*j+3] = y [1][1] / Lx [q] ;
-	    X [2*j+4] = y [2][0] / Lx [r] ;
-	    X [2*j+5] = y [2][1] / Lx [r] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
+	    X [j+2][0] = y [2][0] / Lx [r] ;
+	    X [j+2][1] = y [2][1] / Lx [r] ;
 #else
-	    y [1][0] = X [2*j+2] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [2*j+3] - Lx [p+1] * y [0][1] ;
-	    y [2][0] = X [2*j+4] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [2*j+5] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    X [2*j+2] = y [1][0] ;
-	    X [2*j+3] = y [1][1] ;
-	    X [2*j+4] = y [2][0] ;
-	    X [2*j+5] = y [2][1] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
 #endif
-
 	    for (p += 3, q += 2, r++ ; p < pend ; p++, q++, r++)
 	    {
-		Int i = 2 * Li [p] ;
-		X[i  ] -= Lx[p] * y[0][0] + Lx[q] * y[1][0] + Lx[r] * y[2][0] ;
-		X[i+1] -= Lx[p] * y[0][1] + Lx[q] * y[1][1] + Lx[r] * y[2][1] ;
+		Int i = Li [p] ;
+		X[i][0] -= Lx[p] * y[0][0] + Lx[q] * y[1][0] + Lx[r] * y[2][0] ;
+		X[i][1] -= Lx[p] * y[0][1] + Lx[q] * y[1][1] + Lx[r] * y[2][1] ;
 	    }
-	    j += 3 ;
+	    j += 3 ;	    /* advance to next column of L */
 	}
     }
 }
@@ -344,11 +335,10 @@ static void LSOLVE (PREFIX,2)
 static void LSOLVE (PREFIX,3)
 (
     cholmod_factor *L,
-    cholmod_dense *Y		    /* n-by-3 in row form */
+    double X [ ][3]			/* n-by-3 in row form */
 )
 {
     double *Lx = L->x ;
-    double *X = Y->x ;
     Int *Li = L->i ;
     Int *Lp = L->p ;
     Int *Lnz = L->nz ;
@@ -370,31 +360,30 @@ static void LSOLVE (PREFIX,3)
 	    /* -------------------------------------------------------------- */
 
 	    double y [3] ;
-	    y [0] = X [3*j  ] ;
-	    y [1] = X [3*j+1] ;
-	    y [2] = X [3*j+2] ;
+	    y [0] = X [j][0] ;
+	    y [1] = X [j][1] ;
+	    y [2] = X [j][2] ;
 #ifdef LL
 	    y [0] /= Lx [p] ;
 	    y [1] /= Lx [p] ;
 	    y [2] /= Lx [p] ;
-	    X [3*j  ] = y [0] ;
-	    X [3*j+1] = y [1] ;
-	    X [3*j+2] = y [2] ;
+	    X [j][0] = y [0] ;
+	    X [j][1] = y [1] ;
+	    X [j][2] = y [2] ;
 #elif defined (LD)
-	    X [3*j  ] = y [0] / Lx [p] ;
-	    X [3*j+1] = y [1] / Lx [p] ;
-	    X [3*j+2] = y [2] / Lx [p] ;
+	    X [j][0] = y [0] / Lx [p] ;
+	    X [j][1] = y [1] / Lx [p] ;
+	    X [j][2] = y [2] / Lx [p] ;
 #endif
-
 	    for (p++ ; p < pend ; p++)
 	    {
-		Int i = 3 * Li [p] ;
+		Int i = Li [p] ;
 		double lx = Lx [p] ;
-		X [i  ] -= lx * y [0] ;
-		X [i+1] -= lx * y [1] ;
-		X [i+2] -= lx * y [2] ;
+		X [i][0] -= lx * y [0] ;
+		X [i][1] -= lx * y [1] ;
+		X [i][2] -= lx * y [2] ;
 	    }
-	    j++ ;
+	    j++ ;	/* advance to next column of L */
 
 	}
 	else if (lnz != Lnz [j+2] + 2 || Li [p+2] != j+2)
@@ -406,52 +395,51 @@ static void LSOLVE (PREFIX,3)
 
 	    double y [2][3] ;
 	    Int q = Lp [j+1] ;
-	    y [0][0] = X [3*j  ] ;
-	    y [0][1] = X [3*j+1] ;
-	    y [0][2] = X [3*j+2] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
+	    y [0][2] = X [j][2] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
 	    y [0][2] /= Lx [p] ;
-	    y [1][0] = (X [3*j+3] - Lx [p+1] * y [0][0]) / Lx [q] ;
-	    y [1][1] = (X [3*j+4] - Lx [p+1] * y [0][1]) / Lx [q] ;
-	    y [1][2] = (X [3*j+5] - Lx [p+1] * y [0][2]) / Lx [q] ;
-	    X [3*j  ] = y [0][0] ;
-	    X [3*j+1] = y [0][1] ;
-	    X [3*j+2] = y [0][2] ;
-	    X [3*j+3] = y [1][0] ;
-	    X [3*j+4] = y [1][1] ;
-	    X [3*j+5] = y [1][2] ;
+	    y [1][0] = (X [j+1][0] - Lx [p+1] * y [0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx [p+1] * y [0][1]) / Lx [q] ;
+	    y [1][2] = (X [j+1][2] - Lx [p+1] * y [0][2]) / Lx [q] ;
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j  ][2] = y [0][2] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
 #elif defined (LD)
-	    y [1][0] = X [3*j+3] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [3*j+4] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [3*j+5] - Lx [p+1] * y [0][2] ;
-	    X [3*j  ] = y [0][0] / Lx [p] ;
-	    X [3*j+1] = y [0][1] / Lx [p] ;
-	    X [3*j+2] = y [0][2] / Lx [p] ;
-	    X [3*j+3] = y [1][0] / Lx [q] ;
-	    X [3*j+4] = y [1][1] / Lx [q] ;
-	    X [3*j+5] = y [1][2] / Lx [q] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j  ][2] = y [0][2] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
+	    X [j+1][2] = y [1][2] / Lx [q] ;
 #else
-	    y [1][0] = X [3*j+3] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [3*j+4] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [3*j+5] - Lx [p+1] * y [0][2] ;
-	    X [3*j+3] = y [1][0] ;
-	    X [3*j+4] = y [1][1] ;
-	    X [3*j+5] = y [1][2] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
 #endif
-
 	    for (p += 2, q++ ; p < pend ; p++, q++)
 	    {
-		Int i = 3 * Li [p] ;
+		Int i = Li [p] ;
 		double lx [2] ;
 		lx [0] = Lx [p] ;
 		lx [1] = Lx [q] ;
-		X [i  ] -= lx [0] * y [0][0] + lx [1] * y [1][0] ;
-		X [i+1] -= lx [0] * y [0][1] + lx [1] * y [1][1] ;
-		X [i+2] -= lx [0] * y [0][2] + lx [1] * y [1][2] ;
+		X [i][0] -= lx [0] * y [0][0] + lx [1] * y [1][0] ;
+		X [i][1] -= lx [0] * y [0][1] + lx [1] * y [1][1] ;
+		X [i][2] -= lx [0] * y [0][2] + lx [1] * y [1][2] ;
 	    }
-	    j += 2 ;
+	    j += 2 ;	    /* advance to next column of L */
 
 	}
 	else
@@ -464,71 +452,70 @@ static void LSOLVE (PREFIX,3)
 	    double y [3][3] ;
 	    Int q = Lp [j+1] ;
 	    Int r = Lp [j+2] ;
-	    y [0][0] = X [3*j  ] ;
-	    y [0][1] = X [3*j+1] ;
-	    y [0][2] = X [3*j+2] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
+	    y [0][2] = X [j][2] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
 	    y [0][2] /= Lx [p] ;
-	    y [1][0] = (X[3*j+3] - Lx[p+1] * y[0][0]) / Lx [q] ;
-	    y [1][1] = (X[3*j+4] - Lx[p+1] * y[0][1]) / Lx [q] ;
-	    y [1][2] = (X[3*j+5] - Lx[p+1] * y[0][2]) / Lx [q] ;
-	    y [2][0] = (X[3*j+6] - Lx[p+2] * y[0][0] - Lx[q+1] * y[1][0])/Lx[r];
-	    y [2][1] = (X[3*j+7] - Lx[p+2] * y[0][1] - Lx[q+1] * y[1][1])/Lx[r];
-	    y [2][2] = (X[3*j+8] - Lx[p+2] * y[0][2] - Lx[q+1] * y[1][2])/Lx[r];
-	    X [3*j  ] = y [0][0] ;
-	    X [3*j+1] = y [0][1] ;
-	    X [3*j+2] = y [0][2] ;
-	    X [3*j+3] = y [1][0] ;
-	    X [3*j+4] = y [1][1] ;
-	    X [3*j+5] = y [1][2] ;
-	    X [3*j+6] = y [2][0] ;
-	    X [3*j+7] = y [2][1] ;
-	    X [3*j+8] = y [2][2] ;
+	    y [1][0] = (X [j+1][0] - Lx[p+1] * y[0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx[p+1] * y[0][1]) / Lx [q] ;
+	    y [1][2] = (X [j+1][2] - Lx[p+1] * y[0][2]) / Lx [q] ;
+	    y [2][0] = (X [j+2][0] - Lx[p+2] * y[0][0] - Lx[q+1]*y[1][0])/Lx[r];
+	    y [2][1] = (X [j+2][1] - Lx[p+2] * y[0][1] - Lx[q+1]*y[1][1])/Lx[r];
+	    y [2][2] = (X [j+2][2] - Lx[p+2] * y[0][2] - Lx[q+1]*y[1][2])/Lx[r];
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j  ][2] = y [0][2] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
+	    X [j+2][2] = y [2][2] ;
 #elif defined (LD)
-	    y [1][0] = X [3*j+3] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [3*j+4] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [3*j+5] - Lx [p+1] * y [0][2] ;
-	    y [2][0] = X [3*j+6] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [3*j+7] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    y [2][2] = X [3*j+8] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
-	    X [3*j  ] = y [0][0] / Lx [p] ;
-	    X [3*j+1] = y [0][1] / Lx [p] ;
-	    X [3*j+2] = y [0][2] / Lx [p] ;
-	    X [3*j+3] = y [1][0] / Lx [q] ;
-	    X [3*j+4] = y [1][1] / Lx [q] ;
-	    X [3*j+5] = y [1][2] / Lx [q] ;
-	    X [3*j+6] = y [2][0] / Lx [r] ;
-	    X [3*j+7] = y [2][1] / Lx [r] ;
-	    X [3*j+8] = y [2][2] / Lx [r] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    y [2][2] = X [j+2][2] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j  ][2] = y [0][2] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
+	    X [j+1][2] = y [1][2] / Lx [q] ;
+	    X [j+2][0] = y [2][0] / Lx [r] ;
+	    X [j+2][1] = y [2][1] / Lx [r] ;
+	    X [j+2][2] = y [2][2] / Lx [r] ;
 #else
-	    y [1][0] = X [3*j+3] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [3*j+4] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [3*j+5] - Lx [p+1] * y [0][2] ;
-	    y [2][0] = X [3*j+6] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [3*j+7] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    y [2][2] = X [3*j+8] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
-	    X [3*j+3] = y [1][0] ;
-	    X [3*j+4] = y [1][1] ;
-	    X [3*j+5] = y [1][2] ;
-	    X [3*j+6] = y [2][0] ;
-	    X [3*j+7] = y [2][1] ;
-	    X [3*j+8] = y [2][2] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    y [2][2] = X [j+2][2] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
+	    X [j+2][2] = y [2][2] ;
 #endif
-
 	    for (p += 3, q += 2, r++ ; p < pend ; p++, q++, r++)
 	    {
-		Int i = 3 * Li [p] ;
+		Int i = Li [p] ;
 		double lx [3] ;
 		lx [0] = Lx [p] ;
 		lx [1] = Lx [q] ;
 		lx [2] = Lx [r] ;
-		X [i  ] -= lx[0] * y[0][0] + lx[1] * y[1][0] + lx[2] * y[2][0] ;
-		X [i+1] -= lx[0] * y[0][1] + lx[1] * y[1][1] + lx[2] * y[2][1] ;
-		X [i+2] -= lx[0] * y[0][2] + lx[1] * y[1][2] + lx[2] * y[2][2] ;
+		X [i][0] -= lx[0] * y[0][0] + lx[1] * y[1][0] + lx[2] * y[2][0];
+		X [i][1] -= lx[0] * y[0][1] + lx[1] * y[1][1] + lx[2] * y[2][1];
+		X [i][2] -= lx[0] * y[0][2] + lx[1] * y[1][2] + lx[2] * y[2][2];
 	    }
-	    j += 3 ;
+	    j += 3 ;	    /* advance to next column of L */
 	}
     }
 }
@@ -543,11 +530,10 @@ static void LSOLVE (PREFIX,3)
 static void LSOLVE (PREFIX,4)
 (
     cholmod_factor *L,
-    cholmod_dense *Y		    /* n-by-4 in row form */
+    double X [ ][4]			    /* n-by-4 in row form */
 )
 {
     double *Lx = L->x ;
-    double *X = Y->x ;
     Int *Li = L->i ;
     Int *Lp = L->p ;
     Int *Lnz = L->nz ;
@@ -569,36 +555,35 @@ static void LSOLVE (PREFIX,4)
 	    /* -------------------------------------------------------------- */
 
 	    double y [4] ;
-	    y [0] = X [4*j  ] ;
-	    y [1] = X [4*j+1] ;
-	    y [2] = X [4*j+2] ;
-	    y [3] = X [4*j+3] ;
+	    y [0] = X [j][0] ;
+	    y [1] = X [j][1] ;
+	    y [2] = X [j][2] ;
+	    y [3] = X [j][3] ;
 #ifdef LL
 	    y [0] /= Lx [p] ;
 	    y [1] /= Lx [p] ;
 	    y [2] /= Lx [p] ;
 	    y [3] /= Lx [p] ;
-	    X [4*j  ] = y [0] ;
-	    X [4*j+1] = y [1] ;
-	    X [4*j+2] = y [2] ;
-	    X [4*j+3] = y [3] ;
+	    X [j][0] = y [0] ;
+	    X [j][1] = y [1] ;
+	    X [j][2] = y [2] ;
+	    X [j][3] = y [3] ;
 #elif defined (LD)
-	    X [4*j  ] = y [0] / Lx [p] ;
-	    X [4*j+1] = y [1] / Lx [p] ;
-	    X [4*j+2] = y [2] / Lx [p] ;
-	    X [4*j+3] = y [3] / Lx [p] ;
+	    X [j][0] = y [0] / Lx [p] ;
+	    X [j][1] = y [1] / Lx [p] ;
+	    X [j][2] = y [2] / Lx [p] ;
+	    X [j][3] = y [3] / Lx [p] ;
 #endif
-
 	    for (p++ ; p < pend ; p++)
 	    {
-		Int i = 4 * Li [p] ;
+		Int i = Li [p] ;
 		double lx = Lx [p] ;
-		X [i  ] -= lx * y [0] ;
-		X [i+1] -= lx * y [1] ;
-		X [i+2] -= lx * y [2] ;
-		X [i+3] -= lx * y [3] ;
+		X [i][0] -= lx * y [0] ;
+		X [i][1] -= lx * y [1] ;
+		X [i][2] -= lx * y [2] ;
+		X [i][3] -= lx * y [3] ;
 	    }
-	    j++ ;
+	    j++ ;	/* advance to next column of L */
 
 	}
 	else if (lnz != Lnz [j+2] + 2 || Li [p+2] != j+2)
@@ -610,63 +595,62 @@ static void LSOLVE (PREFIX,4)
 
 	    double y [2][4] ;
 	    Int q = Lp [j+1] ;
-	    y [0][0] = X [4*j  ] ;
-	    y [0][1] = X [4*j+1] ;
-	    y [0][2] = X [4*j+2] ;
-	    y [0][3] = X [4*j+3] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
+	    y [0][2] = X [j][2] ;
+	    y [0][3] = X [j][3] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
 	    y [0][2] /= Lx [p] ;
 	    y [0][3] /= Lx [p] ;
-	    y [1][0] = (X [4*j+4] - Lx [p+1] * y [0][0]) / Lx [q] ;
-	    y [1][1] = (X [4*j+5] - Lx [p+1] * y [0][1]) / Lx [q] ;
-	    y [1][2] = (X [4*j+6] - Lx [p+1] * y [0][2]) / Lx [q] ;
-	    y [1][3] = (X [4*j+7] - Lx [p+1] * y [0][3]) / Lx [q] ;
-	    X [4*j  ] = y [0][0] ;
-	    X [4*j+1] = y [0][1] ;
-	    X [4*j+2] = y [0][2] ;
-	    X [4*j+3] = y [0][3] ;
-	    X [4*j+4] = y [1][0] ;
-	    X [4*j+5] = y [1][1] ;
-	    X [4*j+6] = y [1][2] ;
-	    X [4*j+7] = y [1][3] ;
+	    y [1][0] = (X [j+1][0] - Lx [p+1] * y [0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx [p+1] * y [0][1]) / Lx [q] ;
+	    y [1][2] = (X [j+1][2] - Lx [p+1] * y [0][2]) / Lx [q] ;
+	    y [1][3] = (X [j+1][3] - Lx [p+1] * y [0][3]) / Lx [q] ;
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j  ][2] = y [0][2] ;
+	    X [j  ][3] = y [0][3] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+1][3] = y [1][3] ;
 #elif defined (LD)
-	    y [1][0] = X [4*j+4] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [4*j+5] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [4*j+6] - Lx [p+1] * y [0][2] ;
-	    y [1][3] = X [4*j+7] - Lx [p+1] * y [0][3] ;
-	    X [4*j  ] = y [0][0] / Lx [p] ;
-	    X [4*j+1] = y [0][1] / Lx [p] ;
-	    X [4*j+2] = y [0][2] / Lx [p] ;
-	    X [4*j+3] = y [0][3] / Lx [p] ;
-	    X [4*j+4] = y [1][0] / Lx [q] ;
-	    X [4*j+5] = y [1][1] / Lx [q] ;
-	    X [4*j+6] = y [1][2] / Lx [q] ;
-	    X [4*j+7] = y [1][3] / Lx [q] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [1][3] = X [j+1][3] - Lx [p+1] * y [0][3] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j  ][2] = y [0][2] / Lx [p] ;
+	    X [j  ][3] = y [0][3] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
+	    X [j+1][2] = y [1][2] / Lx [q] ;
+	    X [j+1][3] = y [1][3] / Lx [q] ;
 #else
-	    y [1][0] = X [4*j+4] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [4*j+5] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [4*j+6] - Lx [p+1] * y [0][2] ;
-	    y [1][3] = X [4*j+7] - Lx [p+1] * y [0][3] ;
-	    X [4*j+4] = y [1][0] ;
-	    X [4*j+5] = y [1][1] ;
-	    X [4*j+6] = y [1][2] ;
-	    X [4*j+7] = y [1][3] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [1][3] = X [j+1][3] - Lx [p+1] * y [0][3] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+1][3] = y [1][3] ;
 #endif
-
 	    for (p += 2, q++ ; p < pend ; p++, q++)
 	    {
-		Int i = 4 * Li [p] ;
+		Int i = Li [p] ;
 		double lx [2] ;
 		lx [0] = Lx [p] ;
 		lx [1] = Lx [q] ;
-		X [i  ] -= lx [0] * y [0][0] + lx [1] * y [1][0] ;
-		X [i+1] -= lx [0] * y [0][1] + lx [1] * y [1][1] ;
-		X [i+2] -= lx [0] * y [0][2] + lx [1] * y [1][2] ;
-		X [i+3] -= lx [0] * y [0][3] + lx [1] * y [1][3] ;
+		X [i][0] -= lx [0] * y [0][0] + lx [1] * y [1][0] ;
+		X [i][1] -= lx [0] * y [0][1] + lx [1] * y [1][1] ;
+		X [i][2] -= lx [0] * y [0][2] + lx [1] * y [1][2] ;
+		X [i][3] -= lx [0] * y [0][3] + lx [1] * y [1][3] ;
 	    }
-	    j += 2 ;
+	    j += 2 ;	    /* advance to next column of L */
 
 	}
 	else
@@ -679,88 +663,87 @@ static void LSOLVE (PREFIX,4)
 	    double y [3][4] ;
 	    Int q = Lp [j+1] ;
 	    Int r = Lp [j+2] ;
-	    y [0][0] = X [4*j  ] ;
-	    y [0][1] = X [4*j+1] ;
-	    y [0][2] = X [4*j+2] ;
-	    y [0][3] = X [4*j+3] ;
+	    y [0][0] = X [j][0] ;
+	    y [0][1] = X [j][1] ;
+	    y [0][2] = X [j][2] ;
+	    y [0][3] = X [j][3] ;
 #ifdef LL
 	    y [0][0] /= Lx [p] ;
 	    y [0][1] /= Lx [p] ;
 	    y [0][2] /= Lx [p] ;
 	    y [0][3] /= Lx [p] ;
-	    y [1][0] = (X[4*j+4] - Lx[p+1] * y[0][0]) / Lx [q] ;
-	    y [1][1] = (X[4*j+5] - Lx[p+1] * y[0][1]) / Lx [q] ;
-	    y [1][2] = (X[4*j+6] - Lx[p+1] * y[0][2]) / Lx [q] ;
-	    y [1][3] = (X[4*j+7] - Lx[p+1] * y[0][3]) / Lx [q] ;
-	    y [2][0] = (X[4*j+8] - Lx[p+2] * y[0][0] - Lx[q+1] * y[1][0])/Lx[r];
-	    y [2][1] = (X[4*j+9] - Lx[p+2] * y[0][1] - Lx[q+1] * y[1][1])/Lx[r];
-	    y [2][2] = (X[4*j+10]- Lx[p+2] * y[0][2] - Lx[q+1] * y[1][2])/Lx[r];
-	    y [2][3] = (X[4*j+11]- Lx[p+2] * y[0][3] - Lx[q+1] * y[1][3])/Lx[r];
-	    X [4*j  ] = y [0][0] ;
-	    X [4*j+1] = y [0][1] ;
-	    X [4*j+2] = y [0][2] ;
-	    X [4*j+3] = y [0][3] ;
-	    X [4*j+4] = y [1][0] ;
-	    X [4*j+5] = y [1][1] ;
-	    X [4*j+6] = y [1][2] ;
-	    X [4*j+7] = y [1][3] ;
-	    X [4*j+8] = y [2][0] ;
-	    X [4*j+9] = y [2][1] ;
-	    X [4*j+10]= y [2][2] ;
-	    X [4*j+11]= y [2][3] ;
+	    y [1][0] = (X [j+1][0] - Lx[p+1] * y[0][0]) / Lx [q] ;
+	    y [1][1] = (X [j+1][1] - Lx[p+1] * y[0][1]) / Lx [q] ;
+	    y [1][2] = (X [j+1][2] - Lx[p+1] * y[0][2]) / Lx [q] ;
+	    y [1][3] = (X [j+1][3] - Lx[p+1] * y[0][3]) / Lx [q] ;
+	    y [2][0] = (X [j+2][0] - Lx[p+2] * y[0][0] - Lx[q+1]*y[1][0])/Lx[r];
+	    y [2][1] = (X [j+2][1] - Lx[p+2] * y[0][1] - Lx[q+1]*y[1][1])/Lx[r];
+	    y [2][2] = (X [j+2][2] - Lx[p+2] * y[0][2] - Lx[q+1]*y[1][2])/Lx[r];
+	    y [2][3] = (X [j+2][3] - Lx[p+2] * y[0][3] - Lx[q+1]*y[1][3])/Lx[r];
+	    X [j  ][0] = y [0][0] ;
+	    X [j  ][1] = y [0][1] ;
+	    X [j  ][2] = y [0][2] ;
+	    X [j  ][3] = y [0][3] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+1][3] = y [1][3] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
+	    X [j+2][2] = y [2][2] ;
+	    X [j+2][3] = y [2][3] ;
 #elif defined (LD)
-	    y [1][0] = X [4*j+4] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [4*j+5] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [4*j+6] - Lx [p+1] * y [0][2] ;
-	    y [1][3] = X [4*j+7] - Lx [p+1] * y [0][3] ;
-	    y [2][0] = X [4*j+8] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [4*j+9] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    y [2][2] = X [4*j+10]- Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
-	    y [2][3] = X [4*j+11]- Lx [p+2] * y [0][3] - Lx [q+1] * y [1][3] ;
-	    X [4*j  ] = y [0][0] / Lx [p] ;
-	    X [4*j+1] = y [0][1] / Lx [p] ;
-	    X [4*j+2] = y [0][2] / Lx [p] ;
-	    X [4*j+3] = y [0][3] / Lx [p] ;
-	    X [4*j+4] = y [1][0] / Lx [q] ;
-	    X [4*j+5] = y [1][1] / Lx [q] ;
-	    X [4*j+6] = y [1][2] / Lx [q] ;
-	    X [4*j+7] = y [1][3] / Lx [q] ;
-	    X [4*j+8] = y [2][0] / Lx [r] ;
-	    X [4*j+9] = y [2][1] / Lx [r] ;
-	    X [4*j+10]= y [2][2] / Lx [r] ;
-	    X [4*j+11]= y [2][3] / Lx [r] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [1][3] = X [j+1][3] - Lx [p+1] * y [0][3] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    y [2][2] = X [j+2][2] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
+	    y [2][3] = X [j+2][3] - Lx [p+2] * y [0][3] - Lx [q+1] * y [1][3] ;
+	    X [j  ][0] = y [0][0] / Lx [p] ;
+	    X [j  ][1] = y [0][1] / Lx [p] ;
+	    X [j  ][2] = y [0][2] / Lx [p] ;
+	    X [j  ][3] = y [0][3] / Lx [p] ;
+	    X [j+1][0] = y [1][0] / Lx [q] ;
+	    X [j+1][1] = y [1][1] / Lx [q] ;
+	    X [j+1][2] = y [1][2] / Lx [q] ;
+	    X [j+1][3] = y [1][3] / Lx [q] ;
+	    X [j+2][0] = y [2][0] / Lx [r] ;
+	    X [j+2][1] = y [2][1] / Lx [r] ;
+	    X [j+2][2] = y [2][2] / Lx [r] ;
+	    X [j+2][3] = y [2][3] / Lx [r] ;
 #else
-	    y [1][0] = X [4*j+4] - Lx [p+1] * y [0][0] ;
-	    y [1][1] = X [4*j+5] - Lx [p+1] * y [0][1] ;
-	    y [1][2] = X [4*j+6] - Lx [p+1] * y [0][2] ;
-	    y [1][3] = X [4*j+7] - Lx [p+1] * y [0][3] ;
-	    y [2][0] = X [4*j+8] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
-	    y [2][1] = X [4*j+9] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
-	    y [2][2] = X [4*j+10]- Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
-	    y [2][3] = X [4*j+11]- Lx [p+2] * y [0][3] - Lx [q+1] * y [1][3] ;
-	    X [4*j+4] = y [1][0] ;
-	    X [4*j+5] = y [1][1] ;
-	    X [4*j+6] = y [1][2] ;
-	    X [4*j+7] = y [1][3] ;
-	    X [4*j+8] = y [2][0] ;
-	    X [4*j+9] = y [2][1] ;
-	    X [4*j+10]= y [2][2] ;
-	    X [4*j+11]= y [2][3] ;
+	    y [1][0] = X [j+1][0] - Lx [p+1] * y [0][0] ;
+	    y [1][1] = X [j+1][1] - Lx [p+1] * y [0][1] ;
+	    y [1][2] = X [j+1][2] - Lx [p+1] * y [0][2] ;
+	    y [1][3] = X [j+1][3] - Lx [p+1] * y [0][3] ;
+	    y [2][0] = X [j+2][0] - Lx [p+2] * y [0][0] - Lx [q+1] * y [1][0] ;
+	    y [2][1] = X [j+2][1] - Lx [p+2] * y [0][1] - Lx [q+1] * y [1][1] ;
+	    y [2][2] = X [j+2][2] - Lx [p+2] * y [0][2] - Lx [q+1] * y [1][2] ;
+	    y [2][3] = X [j+2][3] - Lx [p+2] * y [0][3] - Lx [q+1] * y [1][3] ;
+	    X [j+1][0] = y [1][0] ;
+	    X [j+1][1] = y [1][1] ;
+	    X [j+1][2] = y [1][2] ;
+	    X [j+1][3] = y [1][3] ;
+	    X [j+2][0] = y [2][0] ;
+	    X [j+2][1] = y [2][1] ;
+	    X [j+2][2] = y [2][2] ;
+	    X [j+2][3] = y [2][3] ;
 #endif
-
 	    for (p += 3, q += 2, r++ ; p < pend ; p++, q++, r++)
 	    {
-		Int i = 4 * Li [p] ;
+		Int i = Li [p] ;
 		double lx [3] ;
 		lx [0] = Lx [p] ;
 		lx [1] = Lx [q] ;
 		lx [2] = Lx [r] ;
-		X [i  ] -= lx[0] * y[0][0] + lx[1] * y[1][0] + lx[2] * y[2][0] ;
-		X [i+1] -= lx[0] * y[0][1] + lx[1] * y[1][1] + lx[2] * y[2][1] ;
-		X [i+2] -= lx[0] * y[0][2] + lx[1] * y[1][2] + lx[2] * y[2][2] ;
-		X [i+3] -= lx[0] * y[0][3] + lx[1] * y[1][3] + lx[2] * y[2][3] ;
+		X [i][0] -= lx[0] * y[0][0] + lx[1] * y[1][0] + lx[2] * y[2][0];
+		X [i][1] -= lx[0] * y[0][1] + lx[1] * y[1][1] + lx[2] * y[2][1];
+		X [i][2] -= lx[0] * y[0][2] + lx[1] * y[1][2] + lx[2] * y[2][2];
+		X [i][3] -= lx[0] * y[0][3] + lx[1] * y[1][3] + lx[2] * y[2][3];
 	    }
-	    j += 3 ;
+	    j += 3 ;	    /* advance to next column of L */
 	}
     }
 }
@@ -810,10 +793,10 @@ static void LSOLVE (PREFIX,k)
 
     switch (Y->nrow)
     {
-	case 1: LSOLVE (PREFIX,1) (L, Y) ; break ;
-	case 2: LSOLVE (PREFIX,2) (L, Y) ; break ;
-	case 3: LSOLVE (PREFIX,3) (L, Y) ; break ;
-	case 4: LSOLVE (PREFIX,4) (L, Y) ; break ;
+	case 1: LSOLVE (PREFIX,1) (L, Y->x) ; break ;
+	case 2: LSOLVE (PREFIX,2) (L, Y->x) ; break ;
+	case 3: LSOLVE (PREFIX,3) (L, Y->x) ; break ;
+	case 4: LSOLVE (PREFIX,4) (L, Y->x) ; break ;
     }
 
 #else
